/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.examples;

import org.json.simple.JSONObject;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.RemoteException;

public class PublisherApp {

	public static String serializeToJSON(String message, int i) {
		
		JSONObject object = new JSONObject();
		
		object.put("message", message);
		object.put("value", i);
		
		return object.toJSONString();
	}
	
	public static void main(String[] args) {

		Application.This.init(args);
		
		if (Application.This.isAvailable()) {
			System.out.println("Connected");
		}
		
		Application.Publisher publisher = null;
		
		try {
			// Create the publisher not synchronized.
			publisher = Application.Publisher.create("the-publisher");

			System.out.println("Created publisher " + publisher);
			
			// Set the state.
			Application.This.setRunning();

			// Loop on the events.
			int i = 0;
			while (true) {
			
				// Send a message.
				publisher.send(serializeToJSON("a message", i));
				i++;
				
				// Sleep for 1s.
				Thread.sleep(1000);
			}
						
		} catch (RemoteException e) {
			System.out.println("Publisher error");
			
		} catch (InterruptedException e) {
						
		} finally {
			// Terminate the publisher and This.
			publisher.terminate();
			Application.This.terminate();			
		}
		
		System.out.println("Finished the application");
	}

}